@@ -1,15 +1,20 @@
 import logging
 
-import models
-import authentic2.vendor.oath.hotp as hotp
 from django.contrib.auth.models import User
 from django.db import transaction
+from django.conf import settings
 
+import authentic2.vendor.oath.hotp as hotp
+from authentic2.nonce import accept_nonce
+import models
 logger = logging.getLogger('authentic.auth.auth2_oath')
 
-class OATHTOTPBackend:
+NONCE_TIMEOUT = getattr(settings, 'OATH_NONCE_TIMEOUT',
+        getattr(settings, 'NONCE_TIMEOUT', 3600))
 
-    support_oath_otp = True
+class OATHTOTPBackend:
+    supports_object_permissions = False
+    supports_anonymous_user = False
 
     @transaction.commit_on_success()
     def authenticate(self, username, oath_otp, format='dec6'):
@@ -27,6 +32,10 @@ class OATHTOTPBackend:
             raise
 
         if accepted:
+            if not accept_nonce(oath_otp, 'OATH_OTP', NONCE_TIMEOUT):
+                logger.error('already used OTP %r', oath_otp)
+                return None
+
             secret.drift = drift
             return User.objects.get(username=username)
         else:
