@@ -1,14 +1,13 @@
 import re
-import datetime, time
+import time
+import logging
 
 from django.template import RequestContext
-from django.http import HttpResponseRedirect
 from django.contrib.auth import REDIRECT_FIELD_NAME
-from models import MyServiceProvider
 from django.utils.translation import ugettext as _
 from django.shortcuts import render_to_response
-from django.contrib.sessions.models import Session
 from django.contrib import messages
+from django.conf import settings
 
 __redirection_timeout = 1600
 
@@ -37,7 +36,8 @@ def error_page(request, message=None, back=None, logger=None,
     if timer:
         context['redir_timeout'] = __redirection_timeout
         context['next_page'] = back
-    if default_message:
+    display_message = getattr(settings, 'DISPLAY_MESSAGE_ERROR_PAGE', ())
+    if default_message and not display_message:
         messages.add_message(request, messages.ERROR,
             _('An error happened. Report this %s to the administrator.') % \
                 time.strftime("[%Y-%m-%d %a %H:%M:%S]", time.localtime()))
@@ -46,18 +46,6 @@ def error_page(request, message=None, back=None, logger=None,
     return render_to_response('error_authsaml2.html', {'back': back},
             context_instance=context)
 
-def is_sp_configured():
-    cfg = MyServiceProvider.objects.all()
-    if not cfg or not cfg[0]:
-        return False
-    return True
-
-def get_service_provider_settings():
-    cfg = MyServiceProvider.objects.all()
-    if not cfg or not cfg[0]:
-       return None
-    return cfg[0]
-
 # Used to register requested url during SAML redirections
 def register_next_target(request, url=None):
     if url:
@@ -69,7 +57,7 @@ def register_next_target(request, url=None):
     request.session['next'] = next
 
 def get_registered_url(request):
-    if request.session.has_key('next'):
+    if 'next' in request.session:
         return request.session['next']
     return None
 
@@ -77,7 +65,7 @@ def register_request_id(request, request_id):
     request.session['saml_request_id'] = request_id
 
 def check_response_id(request, login):
-    if request.session.has_key('saml_request_id') and \
+    if 'saml_request_id' in request.session and \
         request.session['saml_request_id'] == login.response.inResponseTo:
         return True
     return False
@@ -91,5 +79,5 @@ def save_federation_temp(request, login, attributes=None):
         request.session['attributes'] = attributes
 
 def load_federation_temp(request, login):
-    if request.session.has_key('identity_dump'):
+    if 'identity_dump' in request.session:
         login.setIdentityFromDump(request.session['identity_dump'])
