@@ -1,14 +1,13 @@
 import logging
 
-from django.shortcuts import render_to_response
 from django.contrib.auth import REDIRECT_FIELD_NAME, SESSION_KEY
-from django.utils.http import *
-from django.utils.html import escape
+from django.utils.http import urlencode
 from django.utils.importlib import import_module
 from django.conf import settings
-from django.http import *
+from django.http import HttpResponseRedirect
 
-def redirect_to_login(next, login_url=None, redirect_field_name=REDIRECT_FIELD_NAME, other_keys = {}):
+def redirect_to_login(next, login_url=None,
+        redirect_field_name=REDIRECT_FIELD_NAME, other_keys = {}):
     "Redirects the user to the login page, passing the given 'next' page"
     if not login_url:
         login_url = settings.LOGIN_URL
@@ -28,7 +27,8 @@ def kill_django_sessions(session_key):
     try:
         for key in session_key:
             store = engine.SessionStore(key)
-            logging.debug('Killing session %s of user %s' % (key, store[SESSION_KEY]))
+            logging.debug('Killing session %s of user %s' %
+                    (key, store[SESSION_KEY]))
             store.delete()
     except Exception, e:
         logging.error(e)
