@@ -1,10 +1,24 @@
+import urllib2
+import logging
+
 from django.contrib import admin
 from django.utils.translation import ugettext as _
 from django.conf import settings
 from django.forms import ModelForm
-from django.forms.widgets import MultiWidget
 import django.forms
-from models import *
+from django.contrib import messages
+
+from authentic2.saml.models import LibertyProvider, LibertyServiceProvider
+from authentic2.saml.models import LibertyIdentityProvider, IdPOptionsSPPolicy
+from authentic2.saml.models import SPOptionsIdPPolicy
+from authentic2.saml.models import AuthorizationSPPolicy, AuthorizationAttributeMap
+from authentic2.saml.models import AuthorizationAttributeMapping, LibertyProviderPolicy
+from authentic2.saml.models import LibertySessionDump, LibertyIdentityDump, LibertyFederation
+from authentic2.saml.models import LibertyAssertion, LibertySessionSP, KeyValue
+from authentic2.saml.models import LibertySession
+from authentic2.http_utils import get_url
+
+logger = logging.getLogger(__name__)
 
 class AuthorizationAttributeMapAdmin(admin.ModelAdmin):
     fieldsets = (
@@ -42,10 +56,15 @@ class IdPOptionsSPPolicyAdmin(admin.ModelAdmin):
                         'http_method_for_slo_request'),
                     ('enable_http_method_for_defederation_request',
                         'http_method_for_defederation_request'),
-                    'user_consent',
+                    'force_user_consent',
                     'want_force_authn_request',
                     'want_is_passive_authn_request',
                     'want_authn_request_signed',
+                    'handle_persistent',
+                    'handle_transient',
+                    'back_url',
+                    'accept_slo',
+                    'forward_slo',
                 )
             }),
     )
@@ -73,8 +92,8 @@ class LibertyIdentityProviderInline(admin.StackedInline):
                     'enabled',
                     'enable_following_idp_options_policy',
                     'idp_options_policy',
-                    'enable_following_authorization_policy',
-                    'authorization_policy',
+#                    'enable_following_authorization_policy',
+#                    'authorization_policy',
                 )
             }),
     )
@@ -116,6 +135,23 @@ class LibertyProviderForm(ModelForm):
     class Meta:
         model = LibertyProvider
 
+def update_metadata(modeladmin, request, queryset):
+    updated = []
+    for provider in queryset:
+        if provider.entity_id.startswith('http'):
+            try:
+                data = get_url(provider.entity_id)
+                if data != provider.metadata:
+                    provider.metadata = data
+                    updated.append(provider.entity_id)
+                    provider.save()
+            except (urllib2.URLError, IOError), e:
+                messages.error(request, _('Failure to resolve %s: %s') %
+                        (provider.entity_id, e))
+    if updated:
+        updated = ', '.join(updated)
+        messages.info(request, _('Metadata update for: %s') % updated)
+
 class LibertyProviderAdmin(admin.ModelAdmin):
     form = LibertyProviderForm
     list_display = ('name', 'entity_id', 'protocol_conformance')
@@ -135,12 +171,22 @@ class LibertyProviderAdmin(admin.ModelAdmin):
             LibertyServiceProviderInline,
             LibertyIdentityProviderInline
     ]
+    actions = [ update_metadata ]
+
+
+class LibertyProviderPolicyAdmin(admin.ModelAdmin):
+    inlines = [
+            LibertyServiceProviderInline,
+    ]
+
 
 admin.site.register(IdPOptionsSPPolicy, IdPOptionsSPPolicyAdmin)
-admin.site.register(AuthorizationSPPolicy, AuthorizationSPPolicyAdmin)
-admin.site.register(AuthorizationAttributeMap, AuthorizationAttributeMapAdmin)
-admin.site.register(AuthorizationAttributeMapping, AuthorizationAttributeMappingAdmin)
+admin.site.register(SPOptionsIdPPolicy)
+#admin.site.register(AuthorizationSPPolicy, AuthorizationSPPolicyAdmin)
+#admin.site.register(AuthorizationAttributeMap, AuthorizationAttributeMapAdmin)
+#admin.site.register(AuthorizationAttributeMapping, AuthorizationAttributeMappingAdmin)
 admin.site.register(LibertyProvider, LibertyProviderAdmin)
+admin.site.register(LibertyProviderPolicy, LibertyProviderPolicyAdmin)
 
 if settings.DEBUG:
     admin.site.register(LibertySessionDump)
