@@ -1,5 +1,3 @@
-import os.path
-import time
 import xml.etree.ElementTree as etree
 import hashlib
 import binascii
@@ -11,16 +9,14 @@ from django.db import models
 from django.conf import settings
 from django.contrib.auth.models import User
 from django.core.exceptions import ValidationError
-from django.core.files.storage import FileSystemStorage
 from django.utils.translation import ugettext as _
-from django.core.exceptions import ObjectDoesNotExist, MultipleObjectsReturned
+from django.core.exceptions import ObjectDoesNotExist
 from django.utils.importlib import import_module
 
-from fields import *
+from fields import PickledObjectField, MultiSelectField
+
+from authentic2.idp.models import AttributePolicy
 
-# TODO: add other name formats with lasso next release
-ATTRIBUTE_VALUE_FORMATS = (
-        (lasso.SAML2_ATTRIBUTE_NAME_FORMAT_URI, 'SAMLv2 URI'),)
 
 def metadata_validator(meta):
     provider=lasso.Provider.newFromBuffer(lasso.PROVIDER_ROLE_ANY, meta.encode('utf8'))
@@ -45,6 +41,7 @@ def get_prefered_content(etrees, languages = [None, 'en']):
        else.
     '''
     best = None
+    best_score = -1
     for tree in etrees:
         if best is not None:
             i = ls_find(languages, get_lang(tree))
@@ -124,78 +121,118 @@ BINDING_SSO_IDP = (
     (lasso.SAML2_METADATA_BINDING_ARTIFACT, _('Artifact binding')),
     (lasso.SAML2_METADATA_BINDING_POST, _('POST binding'))
 )
+
+
 HTTP_METHOD = (
     (lasso.HTTP_METHOD_REDIRECT, _('Redirect binding')),
     (lasso.HTTP_METHOD_SOAP, _('SOAP binding'))
 )
 
-USER_CONSENT = (
-    ('urn:oasis:names:tc:SAML:2.0:consent:unspecified', _('Unspecified')),
-    ('urn:oasis:names:tc:SAML:2.0:consent:obtained', _('Obtained')),
-    ('urn:oasis:names:tc:SAML:2.0:consent:prior', _('Prior')),
-    ('urn:oasis:names:tc:SAML:2.0:consent:current-explicit', _('Explicit')),
-    ('urn:oasis:names:tc:SAML:2.0:consent:current-implicit', _('Implicit')),
-    ('urn:oasis:names:tc:SAML:2.0:consent:unavailable', _('Unavailable')),
-    ('urn:oasis:names:tc:SAML:2.0:consent:inapplicable', _('Inapplicable'))
+
+SIGNATURE_VERIFY_HINT = {
+        lasso.PROFILE_SIGNATURE_VERIFY_HINT_MAYBE: _('Let authentic decides which signatures to check'),
+        lasso.PROFILE_SIGNATURE_VERIFY_HINT_FORCE: _('Always check signatures'),
+        lasso.PROFILE_SIGNATURE_VERIFY_HINT_IGNORE: _('Does not check signatures') }
+
+
+class LibertyProviderPolicy(models.Model):
+    name = models.CharField(max_length=64, unique=True)
+    authn_request_signature_check_hint = models.IntegerField(
+            verbose_name=_("How to verify signatures Authentication Request ?"),
+            choices=SIGNATURE_VERIFY_HINT.items(),
+            default=lasso.PROFILE_SIGNATURE_VERIFY_HINT_MAYBE)
+    def __unicode__(self):
+        options = []
+        options.append('AuthnRequest signature: ' + SIGNATURE_VERIFY_HINT[self.authn_request_signature_check_hint])
+        return self.name + ' (%s)' % ', '.join(options)
+
+
+AUTHSAML2_UNAUTH_PERSISTENT = (
+    ('AUTHSAML2_UNAUTH_PERSISTENT_ACCOUNT_LINKING_BY_AUTH',
+        _('Account linking by authentication')),
+    ('AUTHSAML2_UNAUTH_PERSISTENT_CREATE_USER_PSEUDONYMOUS',
+        _('Create new account')),
 )
 
-class LibertyAttributeMap(models.Model):
-    name = models.CharField(max_length = 40, unique = True)
-    def __unicode__(self):
-        return self.name
+AUTHSAML2_UNAUTH_TRANSIENT = (
+    ('AUTHSAML2_UNAUTH_TRANSIENT_ASK_AUTH', _('Ask authentication')),
+    ('AUTHSAML2_UNAUTH_TRANSIENT_OPEN_SESSION', _('Open a session')),
+)
 
-class LibertyAttributeMapping(models.Model):
-    source_attribute_name = models.CharField(max_length = 40)
-    attribute_value_format = models.URLField(choices = ATTRIBUTE_VALUE_FORMATS)
-    attribute_name = models.CharField(max_length = 40)
-    map = models.ForeignKey(LibertyAttributeMap)
 
 class IdPOptionsSPPolicy(models.Model):
-    name = models.CharField(_('name'), max_length=80, unique=True)
+    '''
+        Policies configured as a SAML2 service provider.
+
+        Used to define SAML2 parameters employed with third SAML2 identity
+        providers
+    '''
+    name = models.CharField(_('name'), max_length=200, unique=True)
     enabled = models.BooleanField(verbose_name = _('Enabled'))
     no_nameid_policy = models.BooleanField(
             verbose_name = _("Do not send a nameId Policy"))
-    requested_name_id_format = models.CharField(max_length = 20,
+    requested_name_id_format = models.CharField(
+            verbose_name = _("Requested NameID format"),
+            max_length = 200,
             default = DEFAULT_NAME_ID_FORMAT,
             choices = NAME_ID_FORMATS_CHOICES)
     transient_is_persistent = models.BooleanField(
             verbose_name = \
-            _("This IdP falsely sends a transient NameID \
-            which is in fact persistent"))
+_("This IdP falsely sends a transient NameID which is in fact persistent"))
     allow_create = models.BooleanField(
             verbose_name = _("Allow IdP to create an identity"))
     enable_binding_for_sso_response = models.BooleanField(
             verbose_name = _('Binding for Authnresponse \
             (taken from metadata by the IdP if not enabled)'))
     binding_for_sso_response = models.CharField(
-            max_length = 60, choices = BINDING_SSO_IDP,
-            verbose_name = '',
+            verbose_name = _("Binding for the SSO responses"),
+            max_length = 200, choices = BINDING_SSO_IDP,
             default = lasso.SAML2_METADATA_BINDING_ARTIFACT)
     enable_http_method_for_slo_request = models.BooleanField(
             verbose_name = _('HTTP method for single logout request \
             (taken from metadata if not enabled)'))
     http_method_for_slo_request = models.IntegerField(
-            max_length = 60, choices = HTTP_METHOD,
-            verbose_name = '',
+            verbose_name = _("HTTP binding for the SLO requests"),
+            max_length = 200, choices = HTTP_METHOD,
             default = lasso.HTTP_METHOD_REDIRECT)
     enable_http_method_for_defederation_request = models.BooleanField(
             verbose_name = \
             _('HTTP method for federation termination request \
             (taken from metadata if not enabled)'))
     http_method_for_defederation_request = models.IntegerField(
-            max_length = 60, choices = HTTP_METHOD,
-            verbose_name = '',
+            verbose_name = _("HTTP method for the SLO requests"),
+            max_length = 200, choices = HTTP_METHOD,
             default = lasso.HTTP_METHOD_SOAP)
-    user_consent = models.CharField(
-            max_length = 60, choices = USER_CONSENT,
-            verbose_name = _("Ask user consent"),
-            default = 'urn:oasis:names:tc:SAML:2.0:consent:current-implicit')
+    force_user_consent = models.BooleanField(\
+            verbose_name = \
+                _("Require the user consent be given at account linking"),
+            default=False)
     want_force_authn_request = models.BooleanField(
             verbose_name = _("Force authentication"))
     want_is_passive_authn_request = models.BooleanField(
             verbose_name = _("Passive authentication"))
     want_authn_request_signed = models.BooleanField(
             verbose_name = _("Want AuthnRequest signed"))
+    handle_persistent = models.CharField(
+            max_length=200,
+            verbose_name = _('Behavior with persistent NameID'),
+            choices=AUTHSAML2_UNAUTH_PERSISTENT,
+            default = 'AUTHSAML2_UNAUTH_PERSISTENT_ACCOUNT_LINKING_BY_AUTH')
+    handle_transient = models.CharField(
+            max_length=200,
+            verbose_name = _('Behavior with transient NameID'),
+            choices=AUTHSAML2_UNAUTH_TRANSIENT,
+            default = '')
+    back_url = models.CharField(
+            max_length = 200,
+            default = '/',
+            verbose_name = _('Return URL after a successful authentication'))
+    accept_slo = models.BooleanField(\
+            verbose_name = _("Accept to receive Single Logout requests"),
+            default=True)
+    forward_slo = models.BooleanField(\
+            verbose_name = _("Forward Single Logout requests"),
+            default=True)
 
     class Meta:
         verbose_name = _('identity provider options policy')
@@ -204,6 +241,55 @@ class IdPOptionsSPPolicy(models.Model):
     def __unicode__(self):
         return self.name
 
+
+class SPOptionsIdPPolicy(models.Model):
+    '''
+        Policies configured as a SAML2 identity provider.
+
+        Used to define SAML2 parameters employed with service providers.
+    '''
+    name = models.CharField(_('name'), max_length=80, unique=True)
+    enabled = models.BooleanField(verbose_name = _('Enabled'))
+    prefered_assertion_consumer_binding = models.CharField(
+            verbose_name = _("Prefered assertion consumer binding"),
+            default = 'meta',
+            max_length = 4, choices = ASSERTION_CONSUMER_PROFILES)
+    encrypt_nameid = models.BooleanField(verbose_name = _("Encrypt NameID"))
+    encrypt_assertion = models.BooleanField(
+            verbose_name = _("Encrypt Assertion"))
+    authn_request_signed = models.BooleanField(
+            verbose_name = _("Authentication request signed"))
+    idp_initiated_sso = models.BooleanField(
+            verbose_name = _("Allow IdP initiated SSO"))
+    # XXX: format in the metadata file, should be suffixed with a star to mark
+    # them as special
+    default_name_id_format = models.CharField(max_length = 200,
+            default = DEFAULT_NAME_ID_FORMAT,
+            choices = NAME_ID_FORMATS_CHOICES)
+    accepted_name_id_format = MultiSelectField(
+            verbose_name = _("NameID formats accepted"),
+            max_length=ACCEPTED_NAME_ID_FORMAT_LENGTH,
+            blank=True, choices=NAME_ID_FORMATS_CHOICES)
+    # TODO: add clean method which checks that the LassoProvider we can create
+    # with the metadata file support the SP role
+    # i.e. provider.roles & lasso.PROVIDER_ROLE_SP != 0
+    ask_user_consent = models.BooleanField(
+        verbose_name = _('Ask user for consent when creating a federation'), default = False)
+    accept_slo = models.BooleanField(\
+            verbose_name = _("Accept to receive Single Logout requests"),
+            default=True)
+    forward_slo = models.BooleanField(\
+            verbose_name = _("Forward Single Logout requests"),
+            default=True)
+
+    class Meta:
+        verbose_name = _('service provider options policy')
+        verbose_name_plural = _('service provider options policies')
+
+    def __unicode__(self):
+        return self.name
+
+
 class AuthorizationAttributeMap(models.Model):
     name = models.CharField(max_length = 40, unique = True)
     def __unicode__(self):
@@ -294,34 +380,17 @@ class LibertyServiceProvider(models.Model):
     liberty_provider = models.OneToOneField(LibertyProvider,
             primary_key = True, related_name = 'service_provider')
     enabled = models.BooleanField(verbose_name = _('Enabled'))
-    prefered_assertion_consumer_binding = models.CharField(
-            default = 'meta',
-            max_length = 4, choices = ASSERTION_CONSUMER_PROFILES)
-    encrypt_nameid = models.BooleanField(verbose_name = _("Encrypt NameID"))
-    encrypt_assertion = models.BooleanField(
-            verbose_name = _("Encrypt Assertion"))
-    authn_request_signed = models.BooleanField(
-            verbose_name = _("AuthnRequest signed"))
-    idp_initiated_sso = models.BooleanField(
-            verbose_name = _("Allow IdP initiated SSO"))
-    # Mapping to use to produce attributes in the assertions or in Attribute
-    # requests
-    attribute_map = models.ForeignKey(LibertyAttributeMap,
-            related_name = "service_providers",
-            blank = True, null = True)
-    # XXX: format in the metadata file, should be suffixed with a star to mark
-    # them as special
-    default_name_id_format = models.CharField(max_length = 20,
-            default = DEFAULT_NAME_ID_FORMAT,
-            choices = NAME_ID_FORMATS_CHOICES)
-    accepted_name_id_format = MultiSelectField(
-            max_length=ACCEPTED_NAME_ID_FORMAT_LENGTH,
-            blank=True, choices=NAME_ID_FORMATS_CHOICES)
-    # TODO: add clean method which checks that the LassoProvider we can create
-    # with the metadata file support the SP role
-    # i.e. provider.roles & lasso.PROVIDER_ROLE_SP != 0
-    ask_user_consent = models.BooleanField(
-        verbose_name = _('Ask user for consent when creating a federation'), default = False)
+    enable_following_sp_options_policy = models.BooleanField(verbose_name = \
+        _('The following options policy will apply except if a policy for all service provider is defined.'))
+    sp_options_policy = models.ForeignKey(SPOptionsIdPPolicy, related_name = "sp_options_policy", verbose_name = _('SP Options Policy'), blank=True, null=True)
+    policy = models.ForeignKey(LibertyProviderPolicy,
+            verbose_name=_("Protocol policy"), null=True, default=1)
+    enable_following_attribute_policy = models.BooleanField(verbose_name = \
+        _('The following attribute policy will apply except if a policy for all service provider is defined.'))
+    attribute_policy = models.ForeignKey(AttributePolicy,
+             related_name = "attribute_policy",
+            verbose_name=_("Attribute policy"), null=True, blank=True)
+
 
 # TODO: The choice for requests must be restricted by the IdP metadata
 # The SP then chooses the binding in this list.
@@ -470,6 +539,10 @@ class LibertyFederation(models.Model):
         unique_together = (("name_id_qualifier", "name_id_format",
             "name_id_content", "name_id_sp_name_qualifier"))
 
+    def __unicode__(self):
+        return '<LibertyFederation %s>' % self.__dict__
+
+
 class LibertySession(models.Model):
     """Store the link between a Django session and a Liberty session"""
     django_session_key = models.CharField(max_length = 40)
