@@ -1,11 +1,23 @@
 import time
 import hashlib
 import datetime as dt
+import logging
 
 from django.views.decorators.http import condition
 from django.conf import settings
 from django.http import HttpResponse
 
+from authentic2.saml.saml2utils import filter_attribute_private_key, \
+    filter_element_private_key
+
+
+class CleanLogMessage(logging.Filter):
+    def filter(self, record):
+        record.msg = filter_attribute_private_key(record.msg)
+        record.msg = filter_element_private_key(record.msg)
+        return True
+
+
 class MWT(object):
     """Memoize With Timeout"""
     _caches = {}
@@ -73,7 +85,7 @@ def cache_and_validate(timeout, hashing=hashlib.md5):
                 else:
                     content = f(request, *args, **kwargs)
                     if old_content == content.content:
-                        data = (now, last_modified, etag, old_content)
+                        data = (now, last_modified, etag, mime_type, old_content)
                         return data
             else:
                 content = f(request, *args, **kwargs)
